﻿using System;
using System.Device.Pwm;
using static System.Console;

namespace dsapp
{
    class Program
    {
        #region 字段
        /// <summary>
        /// 高电平持续的最小时间，一般为 500us
        /// </summary>
        static int minMicrosec = 500;

        /// <summary>
        /// 高电平持续的最大时间，一般为 2500us
        /// </summary>
        static int maxMicrosec = 2500;

        /// <summary>
        /// 一度角对应的持续时间，单位为微秒（us）
        /// </summary>
        static double microForAngle = 0d;
        #endregion

        static void Main(string[] args)
        {
            // 创建PWM通道实例
            PwmChannel ch = PwmChannel.Create(0, 0);
            ShowHelps(); //运行后打印一次帮助信息
            bool working = true; //用来跳出循环的标志变量
            while(working)
            {
                Write(">>>");
                // 读取键盘输入的一行文本
                string line = ReadLine();
                // 读取第一个字符
                char first = line[0];
                // 分析命令
                switch(first)
                {
                    case 'h': //显示帮助信息
                        ShowHelps();
                        break;
                    case 'x': //跳出循环，退出程序
                        working = false;
                        break;
                    case 'd': //设置周期，一般是20000us
                        ch.Frequency = ParseFreq(line[1..].Trim());
                        break;
                    case 't': //设置高电平持续的最小时间
                        ParseMinMicrosecond(line[1..].Trim());
                        ComputeMicroToAngle();
                        break;
                    case 'p': //设置高电平持续的最大时间
                        ParseMaxMicrosecond(line[1..].Trim());
                        ComputeMicroToAngle();
                        break;
                    case 'a': //设置要旋转的角度
                        double angle = ParseAngle(line[1..].Trim());
                        ch.DutyCycle = ComputeDuty(angle, ch.Frequency);
                        break;
                    case 'o': //开始发送脉冲
                        ch.Start();
                        break;
                    case 's': //停止发送脉冲
                        ch.Stop();
                        break;
                    default:
                        WriteLine("<<< 无效命令");
                        break;
                }
            }
            ch.Dispose();
        }

        private static int ParseFreq(string s)
        {
            if(int.TryParse(s,out int val) == false)
            {
                val = 20_000;
            }
            return (1_000_000 / val);
        }

        private static void ParseMinMicrosecond(string s)
        {
            if(int.TryParse(s,out int val))
            {
                minMicrosec = val;
            }
        }

        private static void ParseMaxMicrosecond(string s)
        {
            if(int.TryParse(s,out int val))
            {
                maxMicrosec = val;
            }
        }

        static void ShowHelps()
        {
            WriteLine("{0,-15}{1}", "h", "显示帮助信息");
            WriteLine("{0,-15}{1}", "x", "退出");
            WriteLine("{0,-15}{1}", "d n", "设置周期（微秒）");
            WriteLine("{0,-15}{1}", "t n", "高电平起始时间（微秒）");
            WriteLine("{0,-15}{1}", "p n", "高电平结束时间（微秒）");
            WriteLine("{0,-15}{1}", "a n", "角度");
            WriteLine("{0,-15}{1}", "o", "发送脉冲信号");
            WriteLine("{0,-15}{1}", "s", "停止脉冲");
            WriteLine();
        }

        static double ParseAngle(string s)
        {
            if(!double.TryParse(s, out double val))
            {
                val = 90;
            }
            if (val < 0) val = 0;
            if (val > 180) val = 180;
            return val;
        }

        static double ComputeDuty(double angle, int freq)
        {
            return (angle * microForAngle + minMicrosec) / (1_000_000 / freq);
        }

        static void ComputeMicroToAngle()
        {
            microForAngle = (maxMicrosec - minMicrosec) / 180.0d;
        }
    }
}
